"""Module with the definition of common objects for lbh15 package"""
# KEYWORDS

# LEAD CONSTANTS
LEAD_MELTING_TEMPERATURE: float = 600.6  # [K]
LEAD_MELTING_LATENT_HEAT: float = 23.07e3  # [J/kg]
LEAD_BOILING_TEMPERATURE: float = 2021  # [K]
LEAD_VAPORISATION_HEAT: float = 858.6e3  # [J/kg]
LEAD_MOLAR_MASS: float = 0.20720  # [kg/mol]

# BISMUTH CONSTANTS
BISMUTH_MELTING_TEMPERATURE: float = 544.6  # [K]
BISMUTH_MELTING_LATENT_HEAT: float = 53.3e3  # [J/kg]
BISMUTH_BOILING_TEMPERATURE: float = 1831  # [K]
BISMUTH_VAPORISATION_HEAT: float = 856.2e3  # [J/kg]
BISMUTH_MOLAR_MASS: float = 0.20898  # [kg/mol]

# LEAD-BISMUTH-EUTECTIC CONSTANTS
LBE_MELTING_TEMPERATURE: float = 398.0  # [K]
LBE_MELTING_LATENT_HEAT: float = 38.6e3  # [J/kg]
LBE_BOILING_TEMPERATURE: float = 1927  # [K]
LBE_VAPORISATION_HEAT: float = 856.6e3  # [J/kg]
LBE_MOLAR_MASS: float = 0.55*BISMUTH_MOLAR_MASS \
    + 0.45*LEAD_MOLAR_MASS  # [kg/mol]

# OXYGEN CONSTANT
OXYGEN_MOLAR_MASS: float = 0.016  # [kg/mol]
